@@ -656,14 +656,14 @@ static void route_sink_input(struct userdata *u, pa_sink_input *si) {
     pa_assert(u);
     pa_assert(u->do_routing);
 
-    /* Don't override user or application routing requests. */
-    if (si->save_sink || si->sink_requested_by_application)
-        return;
-
     /* Skip this if it is already in the process of being moved anyway */
     if (!si->sink)
         return;
 
+    /* Don't override user or application routing requests. */
+    if (pa_safe_streq(si->sink->name, si->preferred_sink) || si->sink_requested_by_application)
+        return;
+
     auto_filtered_prop = pa_proplist_gets(si->proplist, "module-device-manager.auto_filtered");
     if (auto_filtered_prop)
         auto_filtered = (pa_parse_boolean(auto_filtered_prop) == 1);
@@ -728,10 +728,6 @@ static void route_source_output(struct userdata *u, pa_source_output *so) {
     pa_assert(u);
     pa_assert(u->do_routing);
 
-    /* Don't override user or application routing requests. */
-    if (so->save_source || so->source_requested_by_application)
-        return;
-
     if (so->direct_on_input)
         return;
 
@@ -739,6 +735,10 @@ static void route_source_output(struct userdata *u, pa_source_output *so) {
     if (!so->source)
         return;
 
+    /* Don't override user or application routing requests. */
+    if (pa_safe_streq(so->source->name, so->preferred_source) || so->source_requested_by_application)
+        return;
+
     auto_filtered_prop = pa_proplist_gets(so->proplist, "module-device-manager.auto_filtered");
     if (auto_filtered_prop)
         auto_filtered = (pa_parse_boolean(auto_filtered_prop) == 1);
@@ -1596,7 +1596,7 @@ int pa__init(pa_module*m) {
     }
 
     if (on_rescue) {
-        /* A little bit later than module-stream-restore, a little bit earlier than module-intended-roles, module-rescue-streams, ... */
+        /* A little bit later than module-stream-restore, a little bit earlier than module-intended-roles, ... */
         u->sink_unlink_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SINK_UNLINK], PA_HOOK_LATE+5, (pa_hook_cb_t) sink_unlink_hook_callback, u);
         u->source_unlink_hook_slot = pa_hook_connect(&m->core->hooks[PA_CORE_HOOK_SOURCE_UNLINK], PA_HOOK_LATE+5, (pa_hook_cb_t) source_unlink_hook_callback, u);
     }
