@@ -18,9 +18,7 @@
   General Public License for more details.
 
   You should have received a copy of the GNU Lesser General Public License
-  along with PulseAudio; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
-  USA.
+  along with PulseAudio; if not, see <http://www.gnu.org/licenses/>.
 ***/
 
 #include <sys/types.h>
@@ -40,7 +38,7 @@ int pa_socket_set_rcvbuf(int fd, size_t l);
 int pa_unix_socket_is_stale(const char *fn);
 int pa_unix_socket_remove_stale(const char *fn);
 
-pa_bool_t pa_socket_address_is_local(const struct sockaddr *sa);
-pa_bool_t pa_socket_is_local(int fd);
+bool pa_socket_address_is_local(const struct sockaddr *sa);
+bool pa_socket_is_local(int fd);
 
 #endif
